/*
 * Remove objects from a filesystem.
 *
 * Copyright (C) 2019, 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <stdio.h>

#include <ext2fs/ext2fs.h>

#include "image.h"
#include "session.h"



/* Alternative metadata set by options. */

extern struct metadata md;



/* Remove objects from the filesystem image. */

int _remove(ext2_filsys fs, int argc, char *argv[], int dir_only)
{
    int i;
    const char *path;
    ext2_ino_t ino;

    /* Remove each directory with the given pathname. */

    for (i = 0; i < argc; i++)
    {
        path = argv[i];

        /* Detect missing objects. */

        if ((!*path) || image_find_by_path(fs, path, &ino))
        {
            fprintf(stderr, "Not found: %s\n", path);
            return 1;
        }

        /* Insist on a directory if specified. */

        if (dir_only)
        {
            if (!image_isdir_by_inode(fs, ino))
            {
                fprintf(stderr, "Not a directory: %s\n", path);
                return 1;
            }

            /* Test for an empty directory. */

            if (image_dir_empty_by_inode(fs, ino))
            {
                fprintf(stderr, "Directory not empty: %s\n", path);
                return 1;
            }
        }

        /* Otherwise, insist on a non-directory. */

        else if (image_isdir_by_inode(fs, ino))
        {
            fprintf(stderr, "Cannot remove a directory: %s\n", path);
            return 1;
        }

        /* Unlink the object. */

        if (image_unlink_by_path(fs, path))
        {
            fprintf(stderr, "Could not unlink object: %s\n", path);
            return 1;
        }

        /* Remove the object. */

        if (image_remove_by_inode(fs, ino))
        {
            fprintf(stderr, "Could not remove object: %s\n", path);
            return 1;
        }
    }

    return 0;
}

/* Remove directories from the filesystem image. */

int remove_dirs(ext2_filsys fs, int argc, char *argv[])
{
    return _remove(fs, argc, argv, 1);
}

/* Remove non-directories from the filesystem image. */

int remove_non_dirs(ext2_filsys fs, int argc, char *argv[])
{
    return _remove(fs, argc, argv, 0);
}

/* vim: tabstop=4 expandtab shiftwidth=4
*/
